@@ -27,7 +27,7 @@ from xonsh.inspectors import Inspector
 from xonsh.aliases import Aliases, make_default_aliases
 from xonsh.environ import Env, default_env, locate_binary
 from xonsh.jobs import add_job
-from xonsh.platform import ON_POSIX, ON_WINDOWS, ON_WSL
+from xonsh.platform import ON_POSIX, ON_WINDOWS, ON_WSL, ON_CYGWIN
 from xonsh.proc import (
     PopenThread,
     ProcProxyThread,
@@ -44,6 +44,7 @@ from xonsh.tools import (
     globpath,
     XonshError,
     XonshCalledProcessError,
+    print_color,
 )
 from xonsh.lazyimps import pty, termios, fcntl
 from xonsh.commands_cache import CommandsCache
@@ -179,16 +180,30 @@ def pathsearch(func, s, pymode=False, pathobj=False):
 RE_SHEBANG = LazyObject(lambda: re.compile(r"#![ \t]*(.+?)$"), globals(), "RE_SHEBANG")
 
 
+def is_app_execution_alias(fname):
+    """ App execution aliases behave strangly on windows and python.
+        Here try to detect if a file is an app execution alias
+    """
+    fname = pathlib.Path(fname)
+    return not os.path.exists(fname) and fname.name in os.listdir(fname.parent)
+
+
 def _is_binary(fname, limit=80):
-    with open(fname, "rb") as f:
-        for i in range(limit):
-            char = f.read(1)
-            if char == b"\0":
-                return True
-            if char == b"\n":
-                return False
-            if char == b"":
-                return False
+    try:
+        with open(fname, "rb") as f:
+            for i in range(limit):
+                char = f.read(1)
+                if char == b"\0":
+                    return True
+                if char == b"\n":
+                    return False
+                if char == b"":
+                    return
+    except OSError as e:
+        if ON_WINDOWS and is_app_execution_alias(fname):
+            return True
+        raise e
+
     return False
 
 
@@ -212,7 +227,13 @@ def get_script_subproc_command(fname, args):
     """
     # make sure file is executable
     if not os.access(fname, os.X_OK):
-        raise PermissionError
+        if not ON_CYGWIN:
+            raise PermissionError
+        # explicitly look at all PATH entries for cmd
+        w_path = os.getenv("PATH").split(":")
+        w_fpath = list(map(lambda p: p + os.sep + fname, w_path))
+        if not any(list(map(lambda c: os.access(c, os.X_OK), w_fpath))):
+            raise PermissionError
     if ON_POSIX and not os.access(fname, os.R_OK):
         # on some systems, some important programs (e.g. sudo) will have
         # execute permissions but not read/write permissions. This enables
@@ -390,6 +411,7 @@ class SubprocSpec:
         universal_newlines=False,
         close_fds=False,
         captured=False,
+        env=None,
     ):
         """
         Parameters
@@ -413,6 +435,8 @@ class SubprocSpec:
             The flag for if the subprocess is captured, may be one of:
             False for $[], 'stdout' for $(), 'hiddenobject' for ![], or
             'object' for !().
+        env : dict
+            Replacement environment to run the subporcess in.
 
         Attributes
         ----------
@@ -451,6 +475,13 @@ class SubprocSpec:
         self.universal_newlines = universal_newlines
         self.close_fds = close_fds
         self.captured = captured
+        if env is not None:
+            self.env = {
+                k: v if not (isinstance(v, list)) or len(v) > 1 else v[0]
+                for (k, v) in env.items()
+            }
+        else:
+            self.env = None
         # pure attrs
         self.args = list(cmd)
         self.alias = None
@@ -579,7 +610,8 @@ class SubprocSpec:
 
     def prep_env(self, kwargs):
         """Prepares the environment to use in the subprocess."""
-        denv = builtins.__xonsh__.env.detype()
+        with builtins.__xonsh__.env.swap(self.env) as env:
+            denv = env.detype()
         if ON_WINDOWS:
             # Over write prompt variable as xonsh's $PROMPT does
             # not make much sense for other subprocs
@@ -878,7 +910,7 @@ def _update_last_spec(last):
         last.captured_stderr = last.captured_stdout
 
 
-def cmds_to_specs(cmds, captured=False):
+def cmds_to_specs(cmds, captured=False, envs=None):
     """Converts a list of cmds to a list of SubprocSpec objects that are
     ready to be executed.
     """
@@ -886,11 +918,12 @@ def cmds_to_specs(cmds, captured=False):
     i = 0
     specs = []
     redirects = []
-    for cmd in cmds:
+    for i, cmd in enumerate(cmds):
         if isinstance(cmd, str):
             redirects.append(cmd)
         else:
-            spec = SubprocSpec.build(cmd, captured=captured)
+            env = envs[i] if envs is not None else None
+            spec = SubprocSpec.build(cmd, captured=captured, env=env)
             spec.pipeline_index = i
             specs.append(spec)
             i += 1
@@ -921,7 +954,7 @@ def _should_set_title(captured=False):
     )
 
 
-def run_subproc(cmds, captured=False):
+def run_subproc(cmds, captured=False, envs=None):
     """Runs a subprocess, in its many forms. This takes a list of 'commands,'
     which may be a list of command line arguments or a string, representing
     a special connecting character.  For example::
@@ -937,7 +970,7 @@ def run_subproc(cmds, captured=False):
     if builtins.__xonsh__.env.get("XONSH_TRACE_SUBPROC"):
         print("TRACE SUBPROC: %s" % str(cmds), file=sys.stderr)
 
-    specs = cmds_to_specs(cmds, captured=captured)
+    specs = cmds_to_specs(cmds, captured=captured, envs=envs)
     captured = specs[-1].captured
     if captured == "hiddenobject":
         command = HiddenCommandPipeline(specs)
@@ -982,20 +1015,20 @@ def run_subproc(cmds, captured=False):
         return
 
 
-def subproc_captured_stdout(*cmds):
+def subproc_captured_stdout(*cmds, envs=None):
     """Runs a subprocess, capturing the output. Returns the stdout
     that was produced as a str.
     """
-    return run_subproc(cmds, captured="stdout")
+    return run_subproc(cmds, captured="stdout", envs=envs)
 
 
-def subproc_captured_inject(*cmds):
+def subproc_captured_inject(*cmds, envs=None):
     """Runs a subprocess, capturing the output. Returns a list of
     whitespace-separated strings of the stdout that was produced.
     The string is split using xonsh's lexer, rather than Python's str.split()
     or shlex.split().
     """
-    o = run_subproc(cmds, captured="object")
+    o = run_subproc(cmds, captured="object", envs=envs)
     o.end()
     toks = []
     for line in o:
@@ -1004,26 +1037,26 @@ def subproc_captured_inject(*cmds):
     return toks
 
 
-def subproc_captured_object(*cmds):
+def subproc_captured_object(*cmds, envs=None):
     """
     Runs a subprocess, capturing the output. Returns an instance of
     CommandPipeline representing the completed command.
     """
-    return run_subproc(cmds, captured="object")
+    return run_subproc(cmds, captured="object", envs=envs)
 
 
-def subproc_captured_hiddenobject(*cmds):
+def subproc_captured_hiddenobject(*cmds, envs=None):
     """Runs a subprocess, capturing the output. Returns an instance of
     HiddenCommandPipeline representing the completed command.
     """
-    return run_subproc(cmds, captured="hiddenobject")
+    return run_subproc(cmds, captured="hiddenobject", envs=envs)
 
 
-def subproc_uncaptured(*cmds):
+def subproc_uncaptured(*cmds, envs=None):
     """Runs a subprocess, without capturing the output. Returns the stdout
     that was produced as a str.
     """
-    return run_subproc(cmds, captured=False)
+    return run_subproc(cmds, captured=False, envs=envs)
 
 
 def ensure_list_of_strs(x):
@@ -1353,9 +1386,7 @@ def xonsh_builtins(execer=None):
 
 
 class XonshSession:
-    """All components defining a xonsh session.
-
-    """
+    """All components defining a xonsh session."""
 
     def __init__(self, execer=None, ctx=None):
         """
@@ -1433,6 +1464,8 @@ class XonshSession:
             "execx": "__xonsh__.builtins.execx",
             "compilex": "__xonsh__.builtins.compilex",
             "events": "__xonsh__.builtins.events",
+            "print_color": "__xonsh__.builtins.print_color",
+            "printx": "__xonsh__.builtins.printx",
         }
         for refname, objname in proxy_mapping.items():
             proxy = DynamicAccessProxy(refname, objname)
@@ -1455,6 +1488,8 @@ class XonshSession:
             "compilex",
             "default_aliases",
             "events",
+            "print_color",
+            "printx",
         ]
 
         for name in names:
@@ -1471,6 +1506,7 @@ class _BuiltIns:
         self.execx = None if execer is None else execer.exec
         self.compilex = None if execer is None else execer.compile
         self.events = events
+        self.print_color = self.printx = print_color
 
 
 class DynamicAccessProxy:
