@@ -13,6 +13,7 @@ import collections
 import collections.abc as cabc
 import subprocess
 import platform
+import typing as tp
 
 from xonsh import __version__ as XONSH_VERSION
 from xonsh.lazyasd import LazyObject, lazyobject
@@ -25,6 +26,7 @@ from xonsh.platform import (
     PATH_DEFAULT,
     ON_WINDOWS,
     ON_LINUX,
+    ON_CYGWIN,
     os_environ,
 )
 
@@ -35,6 +37,9 @@ from xonsh.tools import (
     always_false,
     detype,
     ensure_string,
+    is_path,
+    str_to_path,
+    path_to_str,
     is_env_path,
     str_to_env_path,
     env_path_to_str,
@@ -82,6 +87,7 @@ from xonsh.tools import (
     is_str_str_dict,
     to_str_str_dict,
     dict_to_str,
+    to_int_or_none,
 )
 from xonsh.ansi_colors import (
     ansi_color_escape_code_to_name,
@@ -144,10 +150,10 @@ Normal usage is to arm the event handler, then read (not modify) all existing va
 @lazyobject
 def HELP_TEMPLATE():
     return (
-        "{{INTENSE_RED}}{envvar}{{NO_COLOR}}:\n\n"
-        "{{INTENSE_YELLOW}}{docstr}{{NO_COLOR}}\n\n"
-        "default: {{CYAN}}{default}{{NO_COLOR}}\n"
-        "configurable: {{CYAN}}{configurable}{{NO_COLOR}}"
+        "{{INTENSE_RED}}{envvar}{{RESET}}:\n\n"
+        "{{INTENSE_YELLOW}}{docstr}{{RESET}}\n\n"
+        "default: {{CYAN}}{default}{{RESET}}\n"
+        "configurable: {{CYAN}}{configurable}{{RESET}}"
     )
 
 
@@ -324,14 +330,14 @@ class LsColors(cabc.MutableMapping):
         "di": ("BOLD_BLUE",),
         "do": ("BOLD_PURPLE",),
         "ex": ("BOLD_GREEN",),
-        "fi": ("NO_COLOR",),
+        "fi": ("RESET",),
         "ln": ("BOLD_CYAN",),
-        "mh": ("NO_COLOR",),
-        "mi": ("NO_COLOR",),
+        "mh": ("RESET",),
+        "mi": ("RESET",),
         "or": ("BACKGROUND_BLACK", "RED"),
         "ow": ("BLUE", "BACKGROUND_GREEN"),
         "pi": ("BACKGROUND_BLACK", "YELLOW"),
-        "rs": ("NO_COLOR",),
+        "rs": ("RESET",),
         "sg": ("BLACK", "BACKGROUND_YELLOW"),
         "so": ("BOLD_PURPLE",),
         "st": ("WHITE", "BACKGROUND_BLUE"),
@@ -340,7 +346,7 @@ class LsColors(cabc.MutableMapping):
     }
 
     target_value = "target"  # special value to set for ln=target
-    target_color = ("NO_COLOR",)  # repres in color space
+    target_color = ("RESET",)  # repres in color space
 
     def __init__(self, ini_dict: dict = None):
         self._style = self._style_name = None
@@ -399,7 +405,7 @@ class LsColors(cabc.MutableMapping):
                 p.pretty(dict(self))
 
     def is_target(self, key) -> bool:
-        "Return True if key is 'target'"
+        """Return True if key is 'target'"""
         return key in self._targets
 
     def detype(self):
@@ -465,7 +471,7 @@ class LsColors(cabc.MutableMapping):
                     )
                 except Exception as e:
                     print("xonsh:warning:" + str(e), file=sys.stderr)
-                    ini_dict[key] = ("NO_COLOR",)
+                    ini_dict[key] = ("RESET",)
         return cls(ini_dict)
 
     @classmethod
@@ -533,7 +539,8 @@ def ensure_ls_colors_in_env(spec=None, **kwargs):
 ENSURERS = {
     "bool": (is_bool, to_bool, bool_to_str),
     "str": (is_string, ensure_string, ensure_string),
-    "path": (is_env_path, str_to_env_path, env_path_to_str),
+    "path": (is_path, str_to_path, path_to_str),
+    "env_path": (is_env_path, str_to_env_path, env_path_to_str),
     "float": (is_float, float, str),
     "int": (is_int, int, str),
 }
@@ -648,7 +655,9 @@ convert : func
 detype : func
     Function to convert variable from its type to a string representation.
 default
-    Default value for variable.
+    Default value for variable. If set to DefaultNotGiven, raise KeyError
+    instead of returning this default value.  Used for env vars defined
+    outside of Xonsh.
 doc : str
    The environment variable docstring.
 doc_configurable : bool, optional
@@ -665,7 +674,7 @@ doc_store_as_str : bool, optional
 """
 
 # iterates from back
-Var.__new__.__defaults__ = (
+Var.__new__.__defaults__ = (  # type:ignore
     always_true,
     None,
     ensure_string,
@@ -685,7 +694,7 @@ def DEFAULT_VARS():
             is_string,
             ensure_string,
             ensure_string,
-            "",
+            DefaultNotGiven,
             "This is used on Windows to set the title, if available.",
             doc_configurable=False,
         ),
@@ -721,6 +730,23 @@ def DEFAULT_VARS():
             "Places the auto-suggest result as the first option in the completions. "
             "This enables you to tab complete the auto-suggestion.",
         ),
+        "ASYNC_INVALIDATE_INTERVAL": Var(
+            is_float,
+            float,
+            str,
+            0.05,
+            "When ENABLE_ASYNC_PROMPT is True, it may call the redraw frequently. "
+            "This is to group such calls into one that happens within that timeframe. "
+            "The number is set in seconds.",
+        ),
+        "ASYNC_PROMPT_THREAD_WORKERS": Var(
+            is_int,
+            to_int_or_none,
+            str,
+            None,
+            "Define the number of workers used by the ASYC_PROPMT's pool. "
+            "By default it is the same as defined by Python's concurrent.futures.ThreadPoolExecutor class.",
+        ),
         "BASH_COMPLETIONS": Var(
             is_env_path,
             str_to_env_path,
@@ -788,7 +814,7 @@ def DEFAULT_VARS():
             is_bool,
             to_bool,
             bool_to_str,
-            False,
+            True,
             "While tab-completions menu is displayed, press <Enter> to confirm "
             "completion instead of running command. This only affects the "
             "prompt-toolkit shell.",
@@ -873,6 +899,14 @@ def DEFAULT_VARS():
             "The string used to show a shortened directory in a shortened cwd, "
             "e.g. ``'…'``.",
         ),
+        "ENABLE_ASYNC_PROMPT": Var(
+            is_bool,
+            to_bool,
+            bool_to_str,
+            False,
+            "When enabled the prompt is loaded from threads making the shell faster. "
+            "Sections that take long will be updated in the background. ",
+        ),
         "EXPAND_ENV_VARS": Var(
             is_bool,
             to_bool,
@@ -1111,6 +1145,7 @@ def DEFAULT_VARS():
             "'/usr/bin', '/sbin', '/bin', '/usr/games', '/usr/local/games')``",
         ),
         re.compile(r"\w*PATH$"): Var(is_env_path, str_to_env_path, env_path_to_str),
+        re.compile(r"\w*DIRS$"): Var(is_env_path, str_to_env_path, env_path_to_str),
         "PATHEXT": Var(
             is_nonstring_seq_of_strings,
             pathsep_to_upper_seq,
@@ -1284,9 +1319,10 @@ def DEFAULT_VARS():
             is_string,
             ensure_string,
             ensure_string,
-            "",
+            DefaultNotGiven,
             "TERM is sometimes set by the terminal emulator. This is used (when "
-            "valid) to determine whether or not to set the title. Users shouldn't "
+            "valid) to determine whether the terminal emulator can support "
+            "the selected shell, or whether or not to set the title. Users shouldn't "
             "need to set this themselves. Note that this variable should be set as "
             "early as possible in order to ensure it is effective. Here are a few "
             "options:\n\n"
@@ -1304,7 +1340,7 @@ def DEFAULT_VARS():
             is_bool_or_none,
             to_bool_or_none,
             bool_or_none_to_str,
-            True,
+            True if not ON_CYGWIN else False,
             "Whether or not to try to run subrocess mode in a Python thread, "
             "when applicable. There are various trade-offs, which normally "
             "affects only interactive sessions.\n\nWhen True:\n\n"
@@ -1393,7 +1429,7 @@ def DEFAULT_VARS():
             is_string,
             ensure_string,
             ensure_string,
-            "",
+            DefaultNotGiven,
             "Path to the currently active Python environment.",
             doc_configurable=False,
         ),
@@ -1643,7 +1679,7 @@ def DEFAULT_VARS():
             "is prepended whenever stderr is displayed. This may be used in "
             "conjunction with ``$XONSH_STDERR_POSTFIX`` to close out the block."
             "For example, to have stderr appear on a red background, the "
-            'prefix & postfix pair would be "{BACKGROUND_RED}" & "{NO_COLOR}".',
+            'prefix & postfix pair would be "{BACKGROUND_RED}" & "{RESET}".',
         ),
         "XONSH_STDERR_POSTFIX": Var(
             is_string,
@@ -1654,7 +1690,7 @@ def DEFAULT_VARS():
             "is appended whenever stderr is displayed. This may be used in "
             "conjunction with ``$XONSH_STDERR_PREFIX`` to start the block."
             "For example, to have stderr appear on a red background, the "
-            'prefix & postfix pair would be "{BACKGROUND_RED}" & "{NO_COLOR}".',
+            'prefix & postfix pair would be "{BACKGROUND_RED}" & "{RESET}".',
         ),
         "XONSH_STORE_STDIN": Var(
             is_bool,
@@ -1721,7 +1757,8 @@ class Env(cabc.MutableMapping):
     use in a subprocess.
     """
 
-    _arg_regex = None
+    # todo: check this variable is ever used
+    _arg_regex: tp.Optional[str] = None
 
     def __init__(self, *args, **kwargs):
         """If no initial environment is given, os_environ is used."""
@@ -1859,7 +1896,7 @@ class Env(cabc.MutableMapping):
 
     def get_default(self, key, default=None):
         """Gets default for the given key."""
-        if key in self._vars:
+        if key in self._vars and self._vars[key].default is not DefaultNotGiven:
             return self._vars[key].default
         else:
             return default
@@ -1870,7 +1907,12 @@ class Env(cabc.MutableMapping):
         if vd is None:
             vd = Var(default="", doc_default="")
         if vd.doc_default is DefaultNotGiven:
-            dval = pprint.pformat(vd.default)
+            var_default = self._vars.get(key, "<default not set>").default
+            dval = (
+                "not defined"
+                if var_default is DefaultNotGiven
+                else pprint.pformat(var_default)
+            )
             vd = vd._replace(doc_default=dval)
         return vd
 
@@ -1930,12 +1972,11 @@ class Env(cabc.MutableMapping):
     #
 
     def __getitem__(self, key):
-        # remove this block on next release
         if key is Ellipsis:
             return self
         elif key in self._d:
             val = self._d[key]
-        elif key in self._vars:
+        elif key in self._vars and self._vars[key].default is not DefaultNotGiven:
             val = self.get_default(key)
             if is_callable_default(val):
                 val = val(self)
@@ -1986,16 +2027,25 @@ class Env(cabc.MutableMapping):
         """The environment will look up default values from its own defaults if a
         default is not given here.
         """
-        try:
+        if key in self._d or (
+            key in self._vars and self._vars[key].default is not DefaultNotGiven
+        ):
             return self[key]
-        except KeyError:
+        else:
             return default
 
     def rawkeys(self):
         """An iterator that returns all environment keys in their original form.
         This include string & compiled regular expression keys.
         """
-        yield from (set(self._d) | set(self._vars))
+        yield from (
+            set(self._d)
+            | set(
+                k
+                for k in self._vars.keys()
+                if self._vars[k].default is not DefaultNotGiven
+            )
+        )
 
     def __iter__(self):
         for key in self.rawkeys():
@@ -2003,7 +2053,9 @@ class Env(cabc.MutableMapping):
                 yield key
 
     def __contains__(self, item):
-        return item in self._d or item in self._vars
+        return item in self._d or (
+            item in self._vars and self._vars[item].default is not DefaultNotGiven
+        )
 
     def __len__(self):
         return len(self._d)
@@ -2043,7 +2095,7 @@ class Env(cabc.MutableMapping):
         ----------
         name : str
             Environment variable name to register. Typically all caps.
-        type : str, optional,  {'bool', 'str', 'path', 'int', 'float'}
+        type : str, optional,  {'bool', 'str', 'path', 'env_path', 'int', 'float'}
             Variable type. If not one of the available presets, use `validate`,
             `convert`, and `detype` to specify type behavior.
         default : optional
@@ -2071,7 +2123,9 @@ class Env(cabc.MutableMapping):
 
         """
 
-        if (type is not None) and (type in ("bool", "str", "path", "int", "float")):
+        if (type is not None) and (
+            type in ("bool", "str", "path", "env_path", "int", "float")
+        ):
             validate, convert, detype = ENSURERS[type]
 
         if default is not None:
@@ -2079,7 +2133,7 @@ class Env(cabc.MutableMapping):
                 pass
             else:
                 raise ValueError(
-                    "Default value does not match type specified by validate"
+                    f"Default value for {name} does not match type specified by validate"
                 )
 
         self._vars[name] = Var(
