@@ -125,6 +125,7 @@ class History:
         self.last_cmd_out = None
         self.hist_size = None
         self.hist_units = None
+        self.remember_history = True
 
     def __len__(self):
         """Return the number of items in current session."""
@@ -208,6 +209,12 @@ class History:
         """
         pass
 
+    def clear(self):
+        """Clears the history of the current session from both the disk and
+        memory.
+        """
+        pass
+
 #
 # dummy
 #
@@ -250,7 +257,7 @@ cabc = _LazyModule.load('collections', 'collections.abc', 'cabc')
 xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
 xlj = _LazyModule.load('xonsh', 'xonsh.lazyjson', 'xlj')
 uptime = _LazyModule.load('xonsh', 'xonsh.xoreutils.uptime', 'uptime')
-def _xhj_gc_commands_to_rmfiles(hsize, files) -> ([], int):
+def _xhj_gc_commands_to_rmfiles(hsize, files):
     """Return number of units and list of history files to remove to get under the limit,
 
     Parameters:
@@ -437,7 +444,7 @@ class JsonHistoryGC(threading.Thread):
                 # info: file size, closing timestamp, number of commands, filename
                 ts = lj.get("ts", (0.0, None))
                 files.append(
-                    (ts[1] or ts[0], len(lj.sizes["cmds"]) - 1, f, cur_file_size,),
+                    (ts[1] or ts[0], len(lj.sizes["cmds"]) - 1, f, cur_file_size)
                 )
                 lj.close()
                 if xonsh_debug:
@@ -538,6 +545,9 @@ class JsonCommandField(cabc.Sequence):
         return len(self.hist)
 
     def __getitem__(self, key):
+        if not self.hist.remember_history:
+            return ""
+
         size = len(self)
         if isinstance(key, slice):
             return [self[i] for i in range(*key.indices(size))]
@@ -642,6 +652,8 @@ class JsonHistory(History):
         hf : JsonHistoryFlusher or None
             The thread that was spawned to flush history
         """
+        if not self.remember_history:
+            return
         self.buffer.append(cmd)
         self._len += 1  # must come before flushing
         if len(self.buffer) >= self.buffersize:
@@ -664,6 +676,7 @@ class JsonHistory(History):
         hf : JsonHistoryFlusher or None
             The thread that was spawned to flush history
         """
+        # Implicitly covers case of self.remember_history being False.
         if len(self.buffer) == 0:
             return
 
@@ -738,6 +751,21 @@ class JsonHistory(History):
             while self.gc.is_alive():  # while waiting for gc.
                 time.sleep(0.1)  # don't monopolize the thread (or Python GIL?)
 
+    def clear(self):
+        """Clears the current session's history from both memory and disk."""
+
+        # Wipe history from memory. Keep sessionid and other metadata.
+        self.buffer = []
+        self.tss = JsonCommandField("ts", self)
+        self.inps = JsonCommandField("inp", self)
+        self.outs = JsonCommandField("out", self)
+        self.rtns = JsonCommandField("rtn", self)
+        self._len = 0
+        self._skipped = 0
+
+        # Flush empty history object to disk, overwriting previous data.
+        self.flush()
+
 #
 # sqlite
 #
@@ -981,6 +1009,8 @@ class SqliteHistory(History):
         setattr(XH_SQLITE_CACHE, XH_SQLITE_CREATED_SQL_TBL, False)
 
     def append(self, cmd):
+        if not self.remember_history:
+            return
         envs = builtins.__xonsh__.env
         inp = cmd["inp"].rstrip()
         self.inps.append(inp)
@@ -1038,6 +1068,21 @@ class SqliteHistory(History):
             while self.gc.is_alive():
                 continue
 
+    def clear(self):
+        """Clears the current session's history from both memory and disk."""
+        # Wipe memory
+        self.inps = []
+        self.rtns = []
+        self.outs = []
+        self.tss = []
+
+        # Wipe the current session's entries from the database.
+        sql = "DELETE FROM xonsh_history WHERE sessionid = ?"
+        with _xh_sqlite_get_conn(filename=self.filename) as conn:
+            c = conn.cursor()
+            _xh_sqlite_create_history_table(c)
+            c.execute(sql, (str(self.sessionid),))
+
 #
 # main
 #
@@ -1265,7 +1310,18 @@ def _XH_HISTORY_SESSIONS():
     }
 
 
-_XH_MAIN_ACTIONS = {"show", "id", "file", "info", "diff", "gc", "flush"}
+_XH_MAIN_ACTIONS = {
+    "show",
+    "id",
+    "file",
+    "info",
+    "diff",
+    "gc",
+    "flush",
+    "off",
+    "on",
+    "clear",
+}
 
 
 @functools.lru_cache()
@@ -1395,6 +1451,17 @@ def _xh_create_parser():
     # 'flush' subcommand
     subp.add_parser("flush", help="flush the current history to disk")
 
+    # 'off' subcommand
+    subp.add_parser("off", help="history will not be saved for this session")
+
+    # 'on' subcommand
+    subp.add_parser(
+        "on", help="history will be saved for the rest of the session (default)"
+    )
+
+    # 'clear' subcommand
+    subp.add_parser("clear", help="one-time wipe of session history")
+
     return p
 
 
@@ -1458,6 +1525,18 @@ def history_main(
         hf = hist.flush()
         if isinstance(hf, threading.Thread):
             hf.join()
+    elif ns.action == "off":
+        if hist.remember_history:
+            hist.clear()
+            hist.remember_history = False
+            print("History off", file=sys.stderr)
+    elif ns.action == "on":
+        if not hist.remember_history:
+            hist.remember_history = True
+            print("History on", file=sys.stderr)
+    elif ns.action == "clear":
+        hist.clear()
+        print("History cleared", file=sys.stderr)
     else:
         print("Unknown history action {}".format(ns.action), file=sys.stderr)
 
