@@ -18,7 +18,6 @@ import shutil
 import builtins
 import importlib
 import threading
-import subprocess
 import collections
 
 import xonsh.completers.tools as xct
@@ -32,7 +31,6 @@ from xonsh.ansi_colors import (
 from xonsh.prompt.base import multiline_prompt
 from xonsh.tools import (
     print_exception,
-    check_for_partial_string,
     to_bool,
     columnize,
     carriage_return,
@@ -394,7 +392,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         elif len(completions) <= builtins.__xonsh__.env.get("COMPLETION_QUERY_LIMIT"):
             return 2
         msg = "\nDisplay all {} possibilities? ".format(len(completions))
-        msg += "({GREEN}y{NO_COLOR} or {RED}n{NO_COLOR})"
+        msg += "({GREEN}y{RESET} or {RED}n{RESET})"
         self.print_color(msg, end="", flush=True, file=sys.stderr)
         yn = "x"
         while yn not in "yn":
@@ -407,9 +405,9 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         w, h = shutil.get_terminal_size()
         lines = columnize(completions, width=w)
         more_msg = self.format_color(
-            "{YELLOW}==={NO_COLOR} more or "
-            "{PURPLE}({NO_COLOR}q{PURPLE}){NO_COLOR}uit "
-            "{YELLOW}==={NO_COLOR}"
+            "{YELLOW}==={RESET} more or "
+            "{PURPLE}({RESET}q{PURPLE}){RESET}uit "
+            "{YELLOW}==={RESET}"
         )
         while len(lines) > h - 1:
             print("".join(lines[: h - 1]), end="", flush=True, file=sys.stderr)
@@ -464,7 +462,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
             raise ValueError("query completions flag not understood.")
 
     # tab complete on first index too
-    completenames = completedefault
+    completenames = completedefault  # type:ignore
 
     def _load_remaining_input_into_queue(self):
         buf = b""
